/**Copyright or (C) or Copr. GET / ENST, Telecom-Paris, Ludovic Apvrille

ludovic.apvrille AT enst.fr

This software is a computer program whose purpose is to allow the 
edition of TURTLE analysis, design and deployment diagrams, to 
allow the generation of RT-LOTOS or Java code from this diagram, 
and at last to allow the analysis of formal validation traces 
obtained from external tools, e.g. RTL from LAAS-CNRS and CADP 
from INRIA Rhone-Alpes.

This software is governed by the CeCILL  license under French law and
abiding by the rules of distribution of free software.  You can  use, 
modify and/ or redistribute the software under the terms of the CeCILL
license as circulated by CEA, CNRS and INRIA at the following URL
"http://www.cecill.info". 

As a counterpart to the access to the source code and  rights to copy,
modify and redistribute granted by the license, users are provided only
with a limited warranty  and the software's author,  the holder of the
economic rights,  and the successive licensors  have only  limited
liability. 

In this respect, the user's attention is drawn to the risks associated
with loading,  using,  modifying and/or developing or reproducing the
software by the user in light of its specific status of free software,
that may mean  that it is complicated to manipulate,  and  that  also
therefore means  that it is reserved for developers  and  experienced
professionals having in-depth computer knowledge. Users are therefore
encouraged to load and test the software's suitability as regards their
requirements in conditions enabling the security of their systems and/or 
data to be ensured and,  more generally, to use and operate it in the 
same conditions as regards security. 

The fact that you are presently reading this means that you have had
knowledge of the CeCILL license and that you accept its terms.

/**
* Class JPanelFind
* Creation: 05/01/2012
* version 1.0 05/01/2012
* @author Ludovic APVRILLE
* @see
*/

package ui;

//import java.io.*;
import javax.swing.*;
//import javax.swing.event.*;
import javax.swing.table.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.event.*;
//import java.util.*;


import myutil.*;
import ui.*;


public class JPanelFind extends JPanel implements ActionListener, DocumentListener {
	
	private static final int GAP    = 3;
	
    //private JScrollPane jsp;
    private JTextField find;
    private JTextField replace;
    
    private JButton next, previous, replaceOne, replaceAll;
	
	private JCheckBox caseNonDependent;
    
    private MainGUI mgui;

    public JPanelFind(MainGUI _mgui) {

        mgui = _mgui;
        makeComponents();
    }
    
    public void makeComponents() {
        setLayout(new BorderLayout());
        
        
		// Other inputs
		// Inputs from user
		
		// Find
		JPanel jp2 = new JPanel(new GridBagLayout());
		GBHelper pos = new GBHelper();
		jp2.setBorder(new javax.swing.border.TitledBorder("Find"));
        find = new JTextField(5);
        find.getDocument().addDocumentListener(this);
         jp2.add(find, pos.expandW().width(3));
         jp2.add(new Gap(GAP), pos.nextRow());
         next = new JButton(mgui.actions[LEGUIAction.FIND_PREVIOUS]);
         previous = new JButton(mgui.actions[LEGUIAction.FIND]);
         next.setEnabled(false);
         previous.setEnabled(false);
		jp2.add(next, pos.nextRow().align(GridBagConstraints.WEST));
		jp2.add(previous, pos.nextCol().align(GridBagConstraints.WEST));
		caseNonDependent = new JCheckBox("Ignore case");
		caseNonDependent.setSelected(true);
		caseNonDependent.addActionListener(this);
		jp2.add(caseNonDependent, pos.nextRow());
		
		
		// Replace
		JPanel jpr = new JPanel(new GridBagLayout());
		pos = new GBHelper();
		jpr.setBorder(new javax.swing.border.TitledBorder("Replace"));
        replace = new JTextField(5);
        replace.getDocument().addDocumentListener(this);
        jpr.add(replace, pos.expandW().width(3));
        jpr.add(new Gap(GAP), pos.nextRow());
        replaceOne = new JButton(mgui.actions[LEGUIAction.REPLACE_ONE]);
        replaceAll = new JButton(mgui.actions[LEGUIAction.REPLACE_ALL]);
        replaceOne.setEnabled(false);
        replaceAll.setEnabled(false);
		jpr.add(replaceOne, pos.nextRow().expandW());
		jpr.add(replaceAll, pos.nextRow().expandW());
		
		// Panel for both Find and Replace
		JPanel panel2 = new JPanel(new GridBagLayout());
		pos = new GBHelper();
		panel2.add(jp2, pos.nextRow().expandW());
		panel2.add(jpr, pos.nextRow().expandW());
        
		add(panel2, BorderLayout.CENTER);
    }
    
    
    public void insertUpdate(DocumentEvent e) {
		if (e.getDocument() == find.getDocument()) {
				manageFind();
				return;
		}
		
		if (e.getDocument() == replace.getDocument()) {
				manageReplace();
				return;
		}
	
	}
    
	public void removeUpdate(DocumentEvent e) {
		if (e.getDocument() == find.getDocument()) {
				manageFind();
				return;
		}
		
		if (e.getDocument() == replace.getDocument()) {
				manageReplace();
				return;
		}
		
		
	}
	
	public void changedUpdate(DocumentEvent e) {
		if (e.getDocument() == find.getDocument()) {
				manageFind();
		}
		
		if (e.getDocument() == replace.getDocument()) {
				manageReplace();
				return;
		}
	}
    
	private void manageFind() {
		mgui.manageFind(find.getText());
		boolean b = (find.getText().trim().length()>0);
		next.setEnabled(b);
		previous.setEnabled(b);
		replaceOne.setEnabled(b);
		replaceAll.setEnabled(b);
	}
	
	private void manageReplace() {
		mgui.manageReplace(replace.getText());
	}
	
	
	public void	actionPerformed(ActionEvent evt)  {
        
        // Compare the action command to the known actions.
		
		if (evt.getSource() == caseNonDependent) {
			mgui.setMatchCase(!caseNonDependent.isSelected());
			manageFind();
		}
		
	}
	
    
	
    
} // Class