/**Copyright or (C) or Copr. GET / ENST, Telecom-Paris, Ludovic Apvrille

ludovic.apvrille AT enst.fr

This software is a computer program whose purpose is to allow the
edition of Latex documents

This software is governed by the CeCILL  license under French law and
abiding by the rules of distribution of free software.  You can  use, 
modify and/ or redistribute the software under the terms of the CeCILL
license as circulated by CEA, CNRS and INRIA at the following URL
"http://www.cecill.info". 

As a counterpart to the access to the source code and  rights to copy,
modify and redistribute granted by the license, users are provided only
with a limited warranty  and the software's author,  the holder of the
economic rights,  and the successive licensors  have only  limited
liability. 

In this respect, the user's attention is drawn to the risks associated
with loading,  using,  modifying and/or developing or reproducing the
software by the user in light of its specific status of free software,
that may mean  that it is complicated to manipulate,  and  that  also
therefore means  that it is reserved for developers  and  experienced
professionals having in-depth computer knowledge. Users are therefore
encouraged to load and test the software's suitability as regards their
requirements in conditions enabling the security of their systems and/or 
data to be ensured and,  more generally, to use and operate it in the 
same conditions as regards security. 

The fact that you are presently reading this means that you have had
knowledge of the CeCILL license and that you accept its terms.

/**
* Class JDiagramTree
* Tree of latex
* Creation: 11/06/2010
 * Version 1.0 11/06/2010
* @author Ludovic APVRILLE
* @see
*/

package ui.tree;

//import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.event.*;
import javax.swing.tree.*;
import java.util.*;

import myutil.*;
import latexstructure.*;
import ui.*;
import ui.latexedition.*;


public class JDiagramTree extends javax.swing.JTree implements MouseListener, TreeExpansionListener, TreeSelectionListener, Runnable   {
    private boolean toUpdate = false;
    private MainGUI mgui;
	private JPanelInfo jpi;
    private DiagramTreeModel dtm;
    
    //for update
    private Set m_expandedTreePaths = new HashSet();
    private TreePath[] m_selectedTreePaths = new TreePath[0];
    //private boolean m_nodeWasSelected = false;
    
    
    /** Creates new form  */
    public JDiagramTree(MainGUI _mgui, JPanelInfo _jpi) {
        super(new DiagramTreeModel(_mgui));
        mgui = _mgui;
		jpi = _jpi;
        getSelectionModel().setSelectionMode(TreeSelectionModel.SINGLE_TREE_SELECTION);
        setEditable(false);
        addMouseListener(this);
        addTreeExpansionListener(this);
        addTreeSelectionListener(this);
    }
    
    public void reinit() {
        m_expandedTreePaths.clear();
        m_selectedTreePaths = new TreePath[0];
    }
    
    public void toBeUpdated() {
        toUpdate = true;
    }
    
    public void updateNow() {
        if(toUpdate) {
            forceUpdate();
        }
    }
    
    public void forceUpdate() {
		//TraceManager.addDev("Force update");
		if(getSelectionPaths() != null) {
			//TraceManager.addDev("# of paths:" + getSelectionPaths().length);
		}
        toUpdate = false;
		//if (dtm == null) {
			dtm = new DiagramTreeModel(mgui);
			setModel(dtm);
		//}
        
		//revalidate();
		run();
        //update();
    }
    
    public void mousePressed(MouseEvent e) {
        
    }
    
    public void mouseReleased(MouseEvent e) {
        
    }
    
    public void mouseEntered(MouseEvent e) {
        jpi.applyChange();
    }
    
    public void mouseExited(MouseEvent e) {
        //setSelectionPath(null);
        //m_selectedTreePaths = new TreePath[0];
    }
    
    public void mouseClicked(MouseEvent e) {
        
    }
    
    public void run(){
        checkPaths();
        Iterator l_keys = m_expandedTreePaths.iterator();
        TreePath l_path = null;
        while(l_keys.hasNext()){
            try {
                l_path = (TreePath) l_keys.next();
                TreePath parent = l_path.getParentPath();
                //TraceManager.addDev("Path: " + l_path);
                //TraceManager.addDev("Parent path: " + parent);
                if ((l_path.getPathCount() > 1) || (m_expandedTreePaths.contains(parent))) {
					//TraceManager.addDev("Expand path: " + l_path);
					//expandPath(l_path.getParentPath());
                    expandPath(l_path);
                }
            } catch (Exception e) {
                //TraceManager.addDev("Exception " + e.getMessage());
                if (l_path != null) {
                    //TraceManager.addDev("Removing path " + l_path);
                    m_expandedTreePaths.remove(l_path);
                }
            }
        }
        //getSelectionModel().setSelectionPaths(m_selectedTreePaths);
    }
    
    private  void checkPaths() {
		LinkedList<TreePath> toRemove = new LinkedList<TreePath>();
		LinkedList<TreePath> toAdd = new LinkedList<TreePath>();
		
        TreePath l_path = null;
        Iterator l_keys = m_expandedTreePaths.iterator();
		TreePath t_path;
		
        while(l_keys.hasNext()){
            l_path = (TreePath) l_keys.next();
			//TraceManager.addDev("Analyzing: " + l_path);
            if (!isAPathOf(l_path)) {
				//TraceManager.addDev("*************** Not a path!");
				toRemove.add(l_path);
				// Try to find a similar path
				/*t_path = makeSimilarPath(l_path);
				if (t_path == null) {
					TraceManager.addDev("Removing path!");
					toRemove.add(l_path);
				} else {
					TraceManager.addDev("Adding similar path!");
					toAdd.add(t_path);
					toRemove.add(l_path);
				}*/
                //m_expandedTreePaths.remove(l_path);
            }
        }
		
		for(TreePath tp: toRemove) {
			m_expandedTreePaths.remove(tp);
		}
		
		for(TreePath tp: toAdd) {
			m_expandedTreePaths.add(tp);
		}
    }
	
	private TreePath makeSimilarPath(TreePath _path) {
		 if ((dtm ==null) || (_path == null)) {
			 return null;
		 }
		 
		 return dtm.makeSimilarPath(_path);
		 
	}
    
    private boolean isAPathOf(TreePath tp) {
        if ((dtm ==null) || (tp == null)) {
            return false;
        }
        
        Object [] objs = tp.getPath();
        
        if (objs.length == 0) {
            return false;
        }
        
        if (objs[0] != dtm.getRoot()) {
            return false;
        }
        
        int index;
        
        for(int i=0; i<objs.length - 2; i++) {
            index = dtm.getIndexOfChild(objs[i], objs[i+1]);
            if (index == -1) {
                return false;
            }
        }
        
        return true;
        
    }
    
    public synchronized void update(){
        SwingUtilities.invokeLater(this);
    }
    
    public void treeExpanded(TreeExpansionEvent treeExpansionEvent) {
        TreePath tp = treeExpansionEvent.getPath();
        m_expandedTreePaths.add(tp);
		//TraceManager.addDev("New Expand path:" + tp);
        Iterator l_keys = m_expandedTreePaths.iterator();
        while(l_keys.hasNext()){
            TreePath l_path = null;
            try {
                l_path = (TreePath) l_keys.next();
                TreePath parent = l_path.getParentPath();
                if ((l_path.getPathCount() == 1) || (m_expandedTreePaths.contains(parent))) {
					//TraceManager.addDev("New          Expand path:" + l_path);
					
                    expandPath(l_path);
                }
            } catch (Exception e) {
                if (l_path != null) {
                    //TraceManager.addDev("Removing path " + l_path);
                    m_expandedTreePaths.remove(l_path);
                }
            }
        }
        
    }
    
    public void treeCollapsed(TreeExpansionEvent treeExpansionEvent) {
        m_expandedTreePaths.remove(treeExpansionEvent.getPath());
    }
    
    public void valueChanged(TreeSelectionEvent treeSelectionEvent) {
		//System.out.println("Element selected");
		
        if(getSelectionPaths() != null && getSelectionPaths().length >0 ){
            m_selectedTreePaths = getSelectionModel().getSelectionPaths();
        }
        
        TreePath tp = treeSelectionEvent.getNewLeadSelectionPath();
        if (tp == null) {
            return;
        }
        
        Object nodeInfo = tp.getLastPathComponent();
		
		if (nodeInfo instanceof LatexStructure) {
			mgui.selectedElementDocument(0);
		} else if (nodeInfo instanceof LatexElement) {
			mgui.selectedElementDocument(((LatexElement)nodeInfo).getIndex());
		}
		
    }
	
	public void collapseAll() {
		int rc = 0;
		do {
			rc = getRowCount();
			for (int x=rc;x>=0;x--){
				collapseRow(x);
			}
		}
		while (rc != getRowCount()); 
		expandRow(0);
	}
	
	public void expandAll() {
		// Expand all the tree rows...
		int rc = 0;
		do {
			rc = getRowCount();
			for (int x=rc;x>=0;x--){
				expandRow(x);
			}
		}
		while (rc != getRowCount()); 
		// Until there are no more rows to get. i.e the last pass expanded no more new nodes.
	}
    
}
